from abs_nosql_repository_core.document.base_document import BaseDraftDocument
from pydantic import Field, EmailStr
from datetime import datetime


class IntegrationDocument(BaseDraftDocument):
    provider_name: str = Field(..., description="The name of the provider")
    access_token: str = Field(..., description="The access token")
    refresh_token: str = Field(..., description="The refresh token")
    expires_at: datetime = Field(..., description="The expiration date of the access token")

    user_id: int = Field(..., description="The user id")
    email: EmailStr = Field(..., description="The email associated with the integration")
    is_active: bool = Field(True, description="Whether the integration is active and working")

    class Settings:
        name = "integrations"



from abs_nosql_integration_core.service.provider_services import (
    SharepointIntegrationService, 
    GoogleIntegrationService, 
    DocusignIntegrationService,
    OutlookIntegrationService,
    CalendarIntegrationService
)

providers_mapping = {
    "sharepoint": SharepointIntegrationService,
    "google": GoogleIntegrationService,
    "docusign": DocusignIntegrationService,
    "outlook": OutlookIntegrationService,
    "calendar": CalendarIntegrationService,
}


def create_provider_service(provider_name: str, **kwargs):
    return providers_mapping[provider_name](provider_name=provider_name,**kwargs)



from abs_integration_core.schema.common_schema import ResponseSchema
from abs_integration_core.schema.integration_schema import (
    TokenData, 
    Integration, 
    IsConnectedResponse, 
    CreateIntegration, 
    UpdateIntegration,
)
from abs_integration_core.schema.subscription_schema import Subscription, SubscribeRequestSchema

__all__ = [
    "ResponseSchema",
    "TokenData",
    "Integration",
    "IsConnectedResponse",
    "CreateIntegration",
    "UpdateIntegration",
    "Subscription",
    "SubscribeRequestSchema",
]



from abs_integration_core.service.base_services import DocuSignIntegrationBaseService

from abs_nosql_integration_core.repository import IntegrationRepository
from abs_nosql_integration_core.service.nosql_integration_service import (
    IntegrationBaseService,
)
from abs_nosql_integration_core.service.nosql_subscription_service import (
    SubscriptionService,
)
from abs_nosql_integration_core.utils import Encryption
from typing import Optional


class DocusignIntegrationService(
    DocuSignIntegrationBaseService, IntegrationBaseService
):
    def __init__(
        self,
        provider_name: str,
        integration_repository: IntegrationRepository,
        subscription_service: SubscriptionService,
        encryption: Encryption,
        client_id: str,
        client_secret: str,
        redirect_url: Optional[str] = None,
    ):
        super().__init__(
            provider_name=provider_name,
            integration_repository=integration_repository,
            encryption=encryption,
        )

        self.subscription_service = subscription_service

        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_url = redirect_url

        self.scopes = "signature impersonation extended"
        self.docusign_base_oauth_url = "https://account-d.docusign.com/oauth"
        self.docusign_auth_url = f"{self.docusign_base_oauth_url}/auth"

        self.token_url = f"{self.docusign_base_oauth_url}/token"
        self.docusign_api_url = "https://demo.docusign.net/restapi/v2.1"

        self.events = [
            "envelope-sent",
            "envelope-resent",
            "envelope-delivered",
            "envelope-completed",
            "envelope-declined",
            "envelope-voided",
            "recipient-authenticationfailed",
            "recipient-autoresponded",
            "recipient-declined",
            "recipient-delivered",
            "recipient-completed",
            "recipient-sent",
            "recipient-resent",
            "template-created",
            "template-modified",
            "template-deleted",
            "envelope-corrected",
            "envelope-purge",
            "envelope-deleted",
            "envelope-discard",
            "identity-verification-completed",
            "identity-verification-pending",
            "recipient-reassign",
            "recipient-delegate",
            "recipient-finish-later",
            "click-agreed",
            "click-declined",
            "sms-opt-in",
            "sms-opt-out",
            "envelope-created",
            "envelope-removed",
        ]



from abs_integration_core.service.base_services import GoogleIntegrationBaseService

from abs_nosql_integration_core.repository import IntegrationRepository
from abs_nosql_integration_core.service.nosql_integration_service import (
    IntegrationBaseService,
)
from abs_nosql_integration_core.service.nosql_subscription_service import (
    SubscriptionService,
)
from abs_nosql_integration_core.utils import Encryption
from typing import Optional


class GoogleIntegrationService(GoogleIntegrationBaseService, IntegrationBaseService):
    def __init__(
        self,
        provider_name: str,
        integration_repository: IntegrationRepository,
        subscription_service: SubscriptionService,
        encryption: Encryption,
        client_id: str,
        client_secret: str,
        redirect_url: Optional[str] = None,
    ):
        super().__init__(
            provider_name=provider_name,
            integration_repository=integration_repository,
            encryption=encryption,
        )

        self.subscription_service = subscription_service

        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_url = redirect_url

        self.scopes = "https://www.googleapis.com/auth/drive https://www.googleapis.com/auth/userinfo.profile https://www.googleapis.com/auth/userinfo.email"
        self.google_auth_url = "https://accounts.google.com/o/oauth2/v2/auth"
        self.token_url = "https://oauth2.googleapis.com/token"
        self.drive_api_url = "https://www.googleapis.com/drive/v3"



from .docusign_integration_service import DocusignIntegrationService
from .google_integration_service import GoogleIntegrationService
from .sharepoint_integration_service import SharepointIntegrationService
from .outlook_integration_service import OutlookIntegrationService
from .calendar_integration_service import CalendarIntegrationService

__all__ = [
    "DocusignIntegrationService",
    "GoogleIntegrationService",
    "SharepointIntegrationService",
    "OutlookIntegrationService",
    "CalendarIntegrationService",
]



from abs_nosql_integration_core.repository import SubscriptionsRepository
from abs_nosql_repository_core.service.base_service import BaseService
from abs_nosql_integration_core.schema import Subscription
from typing import List, Dict
from abs_nosql_repository_core.schema.base_schema import FilterSchema, LogicalOperator, FieldOperatorCondition, Operator, ListFilter
from abs_exception_core.exceptions import NotFoundError

class SubscriptionService(BaseService):
    def __init__(self, subscription_repository: SubscriptionsRepository):
        super().__init__(subscription_repository)

    async def create(self, schema: Subscription, collection_name: str = None) -> Subscription:
        return await self.repository.create(schema, collection_name)

    async def remove_by_uuid(self, uuid: str) -> Subscription:
        existing_subscription = await self.get_by_attr("uuid", uuid)
        return await super().delete(existing_subscription["id"])
    
    async def list_subscriptions(self, find: Dict) -> List[Subscription]:
        return await super().get_all(ListFilter(**find))

    async def get_user_id_by_subscription_id(self, subscription_id: str) -> int:
        """
        Get the user ID by subscription ID.
        """
        find_query = FilterSchema(
            operator=LogicalOperator.AND,
            conditions=[
                FieldOperatorCondition(
                    field="target_url",
                    operator=Operator.LIKE,
                    value=f".*/{subscription_id}"
                )
            ]
        )

        db_records = await self.get_all(find=ListFilter(filters=find_query, page=1, page_size=1))
        if db_records.get("founds"):
            return db_records["founds"][0]["user_id"]
        else:
            raise NotFoundError("Subscription not found")

    async def get_by_user_and_uuid(self, user_id: int, uuid: str, provider_name: str) -> Subscription:
        """
        Get a subscription by user ID and UUID.
        """
        list_filter = FilterSchema(
            operator=LogicalOperator.AND,
            conditions=[
                FieldOperatorCondition(
                    field="user_id",
                    operator=Operator.EQ,
                    value=user_id
                ),
                FieldOperatorCondition(
                    field="uuid",
                    operator=Operator.EQ,
                    value=uuid
                ),
                FieldOperatorCondition(
                    field="provider_name",
                    operator=Operator.EQ,
                    value=provider_name
                )
            ]
        )

        db_records = await super().get_all(
            find=ListFilter(
                filters=list_filter,
                page=1,
                page_size=1
            )
        )

        if db_records.get("founds"):
            return db_records["founds"][0]
        else:
            raise NotFoundError("Subscription not found")



from abs_nosql_repository_core.document.base_document import BaseDraftDocument
from pydantic import Field
from typing import List, Optional
from datetime import datetime


class SubscriptionDocument(BaseDraftDocument):
    target_url: str = Field(..., description="The target URL to subscribe to")
    site_id: Optional[str] = Field(None, description="The ID of the site")
    resource_id: Optional[str] = Field(None, description="The ID of the resource")
    target_path: str = Field("", description="The target path to subscribe to")
    event_types: List[str] = Field(..., description="The types of events to subscribe to")
    provider_name: str = Field(..., description="The name of the provider")
    expires_at: Optional[datetime] = Field(None, description="The expiration date of the subscription")

    user_id: int = Field(..., description="The ID of the user")
    integration_id: str = Field(..., description="The ID of the integration")

    class Settings:
        name = "subscriptions"



from abs_nosql_repository_core.repository.base_repository import BaseRepository
from abs_nosql_integration_core.model.integration_model import IntegrationDocument
from abs_nosql_integration_core.schema import CreateIntegration, TokenData
from abs_nosql_integration_core.schema.integration_schema import Integration
from abs_nosql_repository_core.schema.base_schema import ListFilter, FilterSchema
from typing import Optional


class IntegrationRepository(BaseRepository):
    def __init__(self):
        super().__init__(document=IntegrationDocument)

    async def create_integration(self, integration_data: CreateIntegration) -> Integration:
        """
        Create a new integration record.
        
        Args:
            integration_data: Integration data including provider_name, access_token, etc.
            
        Returns:
            The created integration object
            
        Raises:
            DuplicatedError: If integration with same provider already exists
        """
        new_integration = Integration(
            provider_name=integration_data.provider_name,
            access_token=integration_data.access_token,
            refresh_token=integration_data.refresh_token,
            expires_at=integration_data.expires_at,
            user_id=integration_data.user_id,
            email=integration_data.email,
        ).model_dump(exclude_none=True)

        existing_integration = await self.get_by_provider_user_email(
            provider_name=integration_data.provider_name,
            user_id=integration_data.user_id,
            email=integration_data.email
        )
        if existing_integration:
            integration_id = existing_integration.get("id",existing_integration.get("_id"),"")
            return await self.update(integration_id, {
                "access_token": integration_data.access_token,
                "refresh_token": integration_data.refresh_token,
                "expires_at": integration_data.expires_at,
            })
        else:
            return await super().create(new_integration)
        
    async def refresh_token(
        self,
        integration_id: str, 
        token_data: TokenData
    ) -> Integration:
        """
        Update token information for a specific integration.
        
        Args:
            provider_name: The integration provider name
            token_data: The data to update
            
        Returns:
            The updated integration object
            
        Raises:
            NotFoundError: If integration doesn't exist
        """
        return await super().update(integration_id, token_data.model_dump(exclude_none=True))
    
    async def get_by_user_and_uuid(self, user_id: int, uuid: str, provider_name: str) -> Optional[Integration]:
        """
        Get integration by UUID.
        
        Args:
            uuid: The integration UUID
            
        Returns:
            The integration object if found, None otherwise
        """
        conditions = [
            {"field": "uuid", "operator": "eq", "value": uuid},
            {"field": "user_id", "operator": "eq", "value": user_id},
            {"field": "provider_name", "operator": "eq", "value": provider_name}
        ]
        
        find_query = FilterSchema(operator="and", conditions=conditions)
        result = await super().get_all(find=ListFilter(filters=find_query, page=1, page_size=1))
        
        if result.get("founds") and len(result["founds"]) > 0:
            return result["founds"][0]
        return None
    
    async def get_by_provider_user_email(
        self,
        provider_name: str,
        user_id: int,
        email: str
    ) -> Optional[Integration]:
        """
        Get integration by provider name, user ID, and email.
        
        Args:
            provider_name: The integration provider name
            user_id: The user ID
            email: The user email
            
        Returns:
            The integration object if found, None otherwise
        """
        conditions = [
            {"field": "provider_name", "operator": "eq", "value": provider_name},
            {"field": "user_id", "operator": "eq", "value": user_id},
            {"field": "email", "operator": "eq", "value": email}
        ]
        
        find_query = FilterSchema(operator="and", conditions=conditions)
        result = await super().get_all(find=ListFilter(filters=find_query, page=1, page_size=1))
        
        if result.get("founds") and len(result["founds"]) > 0:
            return result["founds"][0]
        return None
